@@ -6,12 +6,15 @@ import (
 	"fmt"
 	"io"
 	"os"
+	"strings"
 	"time"
 
 	"k8s.io/klog/v2"
 
 	"github.com/onsi/gomega"
 
+	addonv1alpha1 "github.com/open-cluster-management/api/addon/v1alpha1"
+	addonclient "github.com/open-cluster-management/api/client/addon/clientset/versioned"
 	clusterclient "github.com/open-cluster-management/api/client/cluster/clientset/versioned"
 	operatorclient "github.com/open-cluster-management/api/client/operator/clientset/versioned"
 	workv1client "github.com/open-cluster-management/api/client/work/clientset/versioned"
@@ -20,9 +23,10 @@ import (
 	workapiv1 "github.com/open-cluster-management/api/work/v1"
 	"github.com/open-cluster-management/registration-operator/pkg/helpers"
 	certificatesv1beta1 "k8s.io/api/certificates/v1beta1"
+	coordv1 "k8s.io/api/coordination/v1"
 	corev1 "k8s.io/api/core/v1"
-	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
+	"k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1beta1"
 	"k8s.io/client-go/kubernetes"
@@ -37,6 +41,7 @@ type Tester struct {
 	OperatorClient                   operatorclient.Interface
 	ClusterClient                    clusterclient.Interface
 	WorkClient                       workv1client.Interface
+	AddOnClinet                      addonclient.Interface
 	bootstrapHubSecret               *corev1.Secret
 	EventuallyTimeout                time.Duration
 	EventuallyInterval               time.Duration
@@ -45,6 +50,7 @@ type Tester struct {
 	hubRegistrationDeployment        string
 	hubRegistrationWebhookDeployment string
 	hubWorkWebhookDeployment         string
+	hubPlacementDeployment           string
 	operatorNamespace                string
 	klusterletOperator               string
 }
@@ -62,6 +68,7 @@ func NewTester(kubeconfigPath string) (*Tester, error) {
 		hubRegistrationDeployment:        "cluster-manager-registration-controller",
 		hubRegistrationWebhookDeployment: "cluster-manager-registration-webhook",
 		hubWorkWebhookDeployment:         "cluster-manager-work-webhook",
+		hubPlacementDeployment:           "cluster-manager-placement-controller",
 		operatorNamespace:                "open-cluster-management",
 		klusterletOperator:               "klusterlet",
 	}
@@ -89,6 +96,10 @@ func NewTester(kubeconfigPath string) (*Tester, error) {
 		klog.Errorf("failed to get WorkClient. %v", err)
 		return nil, err
 	}
+	if tester.AddOnClinet, err = addonclient.NewForConfig(tester.ClusterCfg); err != nil {
+		klog.Errorf("failed to get AddOnClinet. %v", err)
+		return nil, err
+	}
 
 	return &tester, nil
 }
@@ -140,8 +151,8 @@ func (t *Tester) CreateKlusterlet(name, clusterName, agentNamespace string) (*op
 			Name: name,
 		},
 		Spec: operatorapiv1.KlusterletSpec{
-			RegistrationImagePullSpec: "quay.io/stolostron/registration:release-2.2",
-			WorkImagePullSpec:         "quay.io/stolostron/work:release-2.2",
+			RegistrationImagePullSpec: "quay.io/open-cluster-management/registration:latest",
+			WorkImagePullSpec:         "quay.io/open-cluster-management/work:latest",
 			ExternalServerURLs: []operatorapiv1.ServerURL{
 				{
 					URL: "https://localhost",
@@ -157,9 +168,12 @@ func (t *Tester) CreateKlusterlet(name, clusterName, agentNamespace string) (*op
 	}
 
 	// create agentNamespace
-	namespace := &v1.Namespace{
+	namespace := &corev1.Namespace{
 		ObjectMeta: metav1.ObjectMeta{
 			Name: agentNamespace,
+			Annotations: map[string]string{
+				"workload.openshift.io/allowed": "management",
+			},
 		},
 	}
 	if _, err := t.KubeClient.CoreV1().Namespaces().Get(context.TODO(), agentNamespace, metav1.GetOptions{}); err != nil {
@@ -388,11 +402,33 @@ func (t *Tester) CheckHubReady() error {
 
 	if _, err := t.KubeClient.AppsV1().Deployments(t.clusterManagerNamespace).
 		Get(context.TODO(), t.hubWorkWebhookDeployment, metav1.GetOptions{}); err != nil {
+	}
+
+	if _, err := t.KubeClient.AppsV1().Deployments(t.clusterManagerNamespace).
+		Get(context.TODO(), t.hubPlacementDeployment, metav1.GetOptions{}); err != nil {
 		return err
 	}
 	return nil
 }
 
+func (t *Tester) CheckClusterManagerStatus() error {
+	cms, err := t.OperatorClient.OperatorV1().ClusterManagers().List(context.TODO(), metav1.ListOptions{})
+	if err != nil {
+		return err
+	}
+	if len(cms.Items) == 0 {
+		return fmt.Errorf("ClusterManager not found")
+	}
+	cm := cms.Items[0]
+	if meta.IsStatusConditionTrue(cm.Status.Conditions, "HubRegistrationDegraded") {
+		return fmt.Errorf("HubRegistration is degraded")
+	}
+	if meta.IsStatusConditionTrue(cm.Status.Conditions, "HubPlacementDegraded") {
+		return fmt.Errorf("HubPlacement is degraded")
+	}
+	return nil
+}
+
 func (t *Tester) CheckKlusterletOperatorReady() error {
 	// make sure klusterlet operator deployment is created
 	_, err := t.KubeClient.AppsV1().Deployments(t.operatorNamespace).
@@ -400,39 +436,22 @@ func (t *Tester) CheckKlusterletOperatorReady() error {
 	return err
 }
 
-func (t *Tester) GetClusterNameFromKlusterlet(klusterletName string) (string, error) {
-	if klusterletName == "" {
-		return "", fmt.Errorf("the klusterlet name should not be null")
-	}
-
-	klusterlet, err := t.OperatorClient.OperatorV1().Klusterlets().Get(context.TODO(),
-		klusterletName, metav1.GetOptions{})
+// GetRandomClusterName gets the clusterName generated by registration randomly.
+// the cluster name is the random name if it has not prefix "e2e-".
+// TODO: get random cluster name from event
+func (t *Tester) GetRandomClusterName() (string, error) {
+	managedClusterList, err := t.ClusterClient.ClusterV1().ManagedClusters().List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return "", err
 	}
 
-	clusterName := klusterlet.Spec.ClusterName
-	if clusterName != "" {
-		return clusterName, nil
+	for _, managedCluster := range managedClusterList.Items {
+		clusterName := managedCluster.Name
+		if !strings.HasPrefix(clusterName, "e2e-") {
+			return clusterName, nil
+		}
 	}
-
-	klusterletNamespace := klusterlet.Spec.Namespace
-	if klusterletNamespace == "" {
-		klusterletNamespace = helpers.KlusterletDefaultNamespace
-	}
-
-	hubKubeconfigSecret, err := t.KubeClient.CoreV1().Secrets(klusterletNamespace).Get(context.TODO(),
-		"hub-kubeconfig-secret", metav1.GetOptions{})
-	if err != nil {
-		return "", err
-	}
-
-	clusterNameByte, ok := hubKubeconfigSecret.Data["cluster-name"]
-	if !ok {
-		return "", fmt.Errorf("there is no cluster-name in secret, %+v", hubKubeconfigSecret)
-	}
-
-	return string(clusterNameByte), nil
+	return "", fmt.Errorf("there is no managedCluster with the random name")
 }
 
 // TODO: only output the details of created resources during e2e
@@ -501,3 +520,66 @@ func (t *Tester) PodLog(podName, nameSpace string, lines int64) (string, error)
 
 	return buf.String(), nil
 }
+
+func (t *Tester) CreateManagedClusterAddOn(managedClusterNamespace, addOnName string) error {
+	_, err := t.AddOnClinet.AddonV1alpha1().ManagedClusterAddOns(managedClusterNamespace).Create(
+		context.TODO(),
+		&addonv1alpha1.ManagedClusterAddOn{
+			ObjectMeta: metav1.ObjectMeta{
+				Namespace: managedClusterNamespace,
+				Name:      addOnName,
+			},
+			Spec: addonv1alpha1.ManagedClusterAddOnSpec{
+				InstallNamespace: addOnName,
+			},
+		},
+		metav1.CreateOptions{},
+	)
+	return err
+}
+
+func (t *Tester) CreateManagedClusterAddOnLease(addOnInstallNamespace, addOnName string) error {
+	if _, err := t.KubeClient.CoreV1().Namespaces().Create(
+		context.TODO(),
+		&corev1.Namespace{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: addOnInstallNamespace,
+			},
+		},
+		metav1.CreateOptions{},
+	); err != nil {
+		return err
+	}
+
+	_, err := t.KubeClient.CoordinationV1().Leases(addOnInstallNamespace).Create(
+		context.TODO(),
+		&coordv1.Lease{
+			ObjectMeta: metav1.ObjectMeta{
+				Name:      addOnName,
+				Namespace: addOnInstallNamespace,
+			},
+			Spec: coordv1.LeaseSpec{
+				RenewTime: &metav1.MicroTime{Time: time.Now()},
+			},
+		},
+		metav1.CreateOptions{},
+	)
+	return err
+}
+
+func (t *Tester) CheckManagedClusterAddOnStatus(managedClusterNamespace, addOnName string) error {
+	addOn, err := t.AddOnClinet.AddonV1alpha1().ManagedClusterAddOns(managedClusterNamespace).Get(context.TODO(), addOnName, metav1.GetOptions{})
+	if err != nil {
+		return err
+	}
+
+	if addOn.Status.Conditions == nil {
+		return fmt.Errorf("there is no conditions in addon %v/%v", managedClusterNamespace, addOnName)
+	}
+
+	if !meta.IsStatusConditionTrue(addOn.Status.Conditions, "Available") {
+		return fmt.Errorf("The addon %v/%v available condition is not true", managedClusterNamespace, addOnName)
+	}
+
+	return nil
+}
